package arzumify.polyenergy.impl;

import arzumify.polyenergy.api.EnergyPresence;
import arzumify.polyenergy.api.EnergyProvider;
import arzumify.polyenergy.api.EnergyReceiver;
import arzumify.presence.matchmaker.Channel;
import arzumify.presence.matchmaker.MatchMaker;
import arzumify.presence.presences.IntegerPresence;

import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

// CoordinateMatchMaker is now implemented using the Presence API

public class CoordinateMatchMaker {
    private static final Channel receivers = new Channel();
    private static final Channel providers = new Channel();
    private static final ConcurrentHashMap<EnergyProvider, Set<EnergyReceiver>> matches = new ConcurrentHashMap<>();
    private static MatchMaker<IntegerPresence, EnergyPresence> matchMaker = new MatchMaker<>();

    public static void reset() {
        matchMaker = new MatchMaker<>();
    }

    public static void addProvider(EnergyProvider provider) {
        matchMaker.Add(provider, providers);
        // Search for receivers within range of this provider
        var receivers = matchMaker.Search(provider, CoordinateMatchMaker.receivers);
        if (receivers != null) {
            for (EnergyPresence receiver : receivers) {
                if (receiver instanceof EnergyReceiver) {
                    provider.exists((EnergyReceiver) receiver);
                }
            }
        }
    }

    public static void addReceiver(EnergyReceiver receiver) {
        matchMaker.Add(receiver, receivers);
        // Search for providers within range of this receiver
        var providers = matchMaker.Search(receiver, CoordinateMatchMaker.providers);
        if (providers != null) {
            for (EnergyPresence provider : providers) {
                if (provider instanceof EnergyProvider) {
                    ((EnergyProvider) provider).exists(receiver);
                    matches.putIfAbsent((EnergyProvider) provider, ConcurrentHashMap.newKeySet());
                    matches.get(provider).add(receiver);
                }
            }
        }
    }

    public static void removeProvider(EnergyProvider provider) {
        matchMaker.Remove(provider, providers);
        if (matches.get(provider) != null) {
            for (EnergyReceiver receiver : matches.get(provider)) {
                receiver.unready(provider);
            }
        }
        matches.remove(provider);
    }

    public static void removeReceiver(EnergyReceiver receiver) {
        matchMaker.Remove(receiver, receivers);
    }
}

